@@ -3,6 +3,7 @@
 """Entry point for cwltool."""
 
 import argparse
+import copy
 import functools
 import io
 import logging
@@ -114,10 +115,12 @@ def _terminate_processes() -> None:
     # we're executing, so it's not safe to use a for loop here.
     while processes_to_kill:
         process = processes_to_kill.popleft()
-        cidfile = [
-            str(arg).split("=")[1] for arg in process.args if "--cidfile" in str(arg)
-        ]
-        if cidfile:
+        if isinstance(process.args, MutableSequence):
+            args = process.args
+        else:
+            args = [process.args]
+        cidfile = [str(arg).split("=")[1] for arg in args if "--cidfile" in str(arg)]
+        if cidfile:  # Try to be nice
             try:
                 with open(cidfile[0]) as inp_stream:
                     p = subprocess.Popen(  # nosec
@@ -129,6 +132,13 @@ def _terminate_processes() -> None:
                         p.kill()
             except FileNotFoundError:
                 pass
+        if process.stdin:
+            process.stdin.close()
+        try:
+            process.wait(10)
+        except subprocess.TimeoutExpired:
+            pass
+        process.kill()  # Always kill, even if we tried with the cidfile
 
 
 def _signal_handler(signum: int, _: Any) -> None:
@@ -219,6 +229,8 @@ def generate_example_input(
             example = ruamel.yaml.comments.CommentedMap()
             if "name" in inptype:
                 comment = '"{}" record type.'.format(inptype["name"])
+            else:
+                comment = "Anonymous record type."
             for field in cast(List[CWLObjectType], inptype["fields"]):
                 value, f_comment = generate_example_input(field["type"], None)
                 example.insert(0, shortname(cast(str, field["name"])), value, f_comment)
@@ -652,12 +664,14 @@ def setup_schema(
             ext11 = res.read().decode("utf-8")
         use_custom_schema("v1.0", "http://commonwl.org/cwltool", ext10)
         use_custom_schema("v1.1", "http://commonwl.org/cwltool", ext11)
+        use_custom_schema("v1.2", "http://commonwl.org/cwltool", ext11)
         use_custom_schema("v1.2.0-dev1", "http://commonwl.org/cwltool", ext11)
         use_custom_schema("v1.2.0-dev2", "http://commonwl.org/cwltool", ext11)
         use_custom_schema("v1.2.0-dev3", "http://commonwl.org/cwltool", ext11)
     else:
         use_standard_schema("v1.0")
         use_standard_schema("v1.1")
+        use_standard_schema("v1.2")
         use_standard_schema("v1.2.0-dev1")
         use_standard_schema("v1.2.0-dev2")
         use_standard_schema("v1.2.0-dev3")
@@ -680,11 +694,14 @@ class ProvLogFormatter(logging.Formatter):
         return with_msecs
 
 
+ProvOut = Union[io.TextIOWrapper, WritableBagFile]
+
+
 def setup_provenance(
     args: argparse.Namespace,
     argsl: List[str],
     runtimeContext: RuntimeContext,
-) -> Union[io.TextIOWrapper, WritableBagFile]:
+) -> Tuple[ProvOut, "logging.StreamHandler[ProvOut]"]:
     if not args.compute_checksum:
         _logger.error("--provenance incompatible with --no-compute-checksum")
         raise ArgumentException()
@@ -705,7 +722,7 @@ def setup_provenance(
         # Log cwltool command line options to provenance file
         _logger.info("[cwltool] %s %s", sys.argv[0], " ".join(argsl))
     _logger.debug("[cwltool] Arguments: %s", args)
-    return log_file_io
+    return log_file_io, prov_log_handler
 
 
 def setup_loadingContext(
@@ -935,6 +952,9 @@ def print_targets(
                 + "\n"
             )
     if "steps" in tool.tool:
+        loading_context = copy.copy(loading_context)
+        loading_context.requirements = tool.requirements
+        loading_context.hints = tool.hints
         _logger.info("%s steps targets:", prefix[:-1])
         for t in tool.tool["steps"]:
             stdout.write(f"  {prefix}{shortname(t['id'])}\n")
@@ -983,7 +1003,7 @@ def main(
         coloredlogs.install(logger=_logger, stream=stderr)
         stderr_handler = _logger.handlers[-1]
     workflowobj = None
-    prov_log_handler = None  # type: Optional[logging.StreamHandler]
+    prov_log_handler: Optional[logging.StreamHandler[ProvOut]] = None
     try:
         if args is None:
             if argsl is None:
@@ -1051,7 +1071,9 @@ def main(
             if argsl is None:
                 raise Exception("argsl cannot be None")
             try:
-                prov_log_stream = setup_provenance(args, argsl, runtimeContext)
+                prov_log_stream, prov_log_handler = setup_provenance(
+                    args, argsl, runtimeContext
+                )
             except ArgumentException:
                 return 1
 
